/**
 * Title:        <p>
 * Description:  <p>
 * Copyright:    Copyright (c) <p>
 * Company:      <p>
 * @author
 * @version 1.0
 */
package gaiacrtn.config;

import gaiacrtn.config.PropertiesApp;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;


/** User properties list.
 * This list is used to keep track of user settings. The user properties list is
 * stored in the user home directory.
 */
public class PropertiesUser extends Properties {
   private File _fileProperties;
   private String _strHeader;

   /** Creates user properties using default naming schema.
    * The user properties filename is PropertiesApp.appName() + ".user.properties".
    * For example, if the application name is set to "DiffWord", then the
    * user properties filename will be "DiffWord.user.properties" and will
    * be placed in the user home directory.
    */
   static public PropertiesUser create( Properties prpDefault ) {
      return new PropertiesUser(
            PropertiesApp.appName() + ".user.properties",
            PropertiesApp.appName() + PropertiesApp.appVersion() + " User Properties",
            prpDefault );
   }

   /** Constructs a user properties list.
    * @param strFilename Filename where the properties are stored. strFilename is relative
    *                    to the user home directory. Must no be null. There "myapp.properties"
    *                    will be stored in "/users/home/toto/myapp.properties"
    *                    (or "G:\Documents and Settings\toto").
    * @param strHeader Header that is added at the beginning of the properties file.
    * @param prpDefault Default properties set. If a properties does not exist in the set,
    *                   then its value is taken from the prpDefault set. May be null.
    */
   public PropertiesUser( String strFilename, String strHeader, Properties prpDefault ) {
      final File fileUserHome = new File( System.getProperty( "user.home" ) );
      _fileProperties = new File( fileUserHome, strFilename );
      _strHeader = strHeader;
      load();
   }

   /** Load the properties list.
    *
    */
   public void load() {
      try {
         super.load( new FileInputStream( _fileProperties ) );
      } catch (IOException e) {
//      System.err.println( "Failed to read user properties: " + e );
      }
   }

   /** Save the properties list.
    *
    */
   public void save() {
      try {
         super.store( new FileOutputStream( _fileProperties ), _strHeader );
      } catch (IOException e) {
         System.err.println( "Failed to write user properties: " + e );
      }
   }


   public void setPrp( String strKey, String strValue ) {
      setProperty( strKey, strValue );
   }

   public String getPrp( String strKey ) {
      return getProperty( strKey );
   }

   public String getPrp( String strKey, String strDefaultValue ) {
      return getProperty( strKey, strDefaultValue );
   }

   public boolean getBoolPrp( String strKey ) {
      return Boolean.valueOf( getPrp( strKey ) ).booleanValue();
   }

   public boolean getBoolPrp( String strKey,
                              boolean bDefaultValue ) {
      return Boolean.valueOf(
            getPrp( strKey, String.valueOf( bDefaultValue ) ) ).booleanValue();
   }

   public int getIntPrp( String strKey ) {
      return Integer.valueOf( getPrp( strKey ) ).intValue();
   }

   public int getIntPrp( String strKey, int nDefaultValue ) {
      return Integer.valueOf(
            getPrp( strKey, String.valueOf( nDefaultValue ) ) ).intValue();
   }

}