/**
 * Title:        Diff Word<p>
 * Description:  A diff that is not line based but word based.<p>
 * Copyright:    Copyright (c) Baptiste Lepilleur<p>
 * Company:      Gaia Creation<p>
 * @author Baptiste Lepilleur
 * @version 1.0
 */

package gaiacrtn.diffword;

import gaiacrtn.swing.*;
import gaiacrtn.swing.text.DocumentHelper;
import gaiacrtn.util.*;
import gaiacrtn.diffword.ui.model.ModelFilename;
import gaiacrtn.diffword.ui.ModelString2TextFieldBinder;
import gaiacrtn.diffword.ui.DwPanelFileSelect;
import gaiacrtn.config.PropertiesApp;
import gaiacrtn.config.PropertiesUser;
import gaiacrtn.config.ResourceApp;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLDocument;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.StringTokenizer;

public class DiffWordApp {
   final static String PRP_LOCALE_FORCE = "locale.force";
   final static String PRP_LOCALE_NAME = "locale.name";
   final static String PRP_DIFF_SAVE_HTML_FILENAME = "diff.save.html.filename";
   final static String PRP_DIFF_LOAD_FILENAME0 = "diff.load.filename0";
   final static String PRP_DIFF_LOAD_FILENAME1 = "diff.load.filename1";

   PropertiesUser _prpUser;

   ModelFilename _modelFilename0;
   ModelFilename _modelFilename1;

   void setupApp() {
      PropertiesApp.setPrp( PropertiesApp.APP_NAME, "DiffWord" );
      PropertiesApp.setPrp( PropertiesApp.APP_VERSION, "1.0" );
      PropertiesApp.setPrp( PropertiesApp.APP_HOME,
                            System.getProperty( "user.dir" ) );

      // load user properties
      _prpUser = PropertiesUser.create( null );
      _prpUser.load();
      if ( _prpUser.getBoolPrp( PRP_LOCALE_FORCE, false ) ) {
         String strLocaleName = _prpUser.getPrp( PRP_LOCALE_NAME );
         StringTokenizer tokenizer = new StringTokenizer( strLocaleName, "_" );
         if ( tokenizer.countTokens() < 1 )
            System.err.println( "Bad locale name in user properties!" );
         else {
            String strLanguage = tokenizer.nextToken();
            String strCountry = "";
            String strVariant = "";
            if ( tokenizer.hasMoreTokens() )
               strCountry = tokenizer.nextToken();
            if ( tokenizer.hasMoreTokens() )
               strVariant = tokenizer.nextToken();
            Locale.setDefault( new Locale( strLanguage, strCountry, strVariant ) );
         }
      }

      // load application resources
      try {
         ResourceApp.setResApp( ResourceApp.create() );
      } catch (MissingResourceException e) {
         System.err.println( "Failed to load resource file:\n" + e );
         quitInitFail();
      }

      // load xmenu application definition
      try {
         MenuDefFactory factoryMenuDef = new MenuDefFactoryXMenu(
               ResourceApp.appFile( "resources/" + PropertiesApp.appName() +
                                    "-xmenu.xml" ) );
         ResourceApp.setAppMenuDefFactory( factoryMenuDef );
      } catch (IOException e) {
         System.err.println( "Failed to load xmenu application definition:\n" +
                             e );
         quitInitFail();
      }

      // creates application command holder
      ResourceApp.setAppCommandHolder( new CommandHolder() );
   }

   public DiffWordApp() {
      setupApp();
      setupUserProperties();

      setupCommands();

      JFrame frameMain = new JFrame( "DiffWord" );
      JPanel panelFileSel0 = new DwPanelFileSelect( _modelFilename0 );
      JPanel panelFileSel1 = new DwPanelFileSelect( _modelFilename1 );
      JPanel panelContent = new JPanel();
      JButton buttonShowChange = new JButton();
      ResourceApp.cmdHolder.setup( buttonShowChange, "diff.show" );

//    panelContent.setLayout( new BorderLayout() );
      panelContent.setLayout( new FlowLayout() );
      panelContent.setBorder( BorderFactory.createEmptyBorder( 4, 4, 4, 4 ) );
//    panelContent.add( panelFileSel0, BorderLayout.WEST );
//    panelContent.add( panelFileSel1, BorderLayout.EAST );
      panelContent.add( panelFileSel0 );
      panelContent.add( panelFileSel1 );
      panelContent.add( buttonShowChange );

      frameMain.setContentPane( panelContent );

      frameMain.setJMenuBar( MenuMaker.makeMenuBar( "main.menubar" ) );

      frameMain.addWindowListener( new WindowAdapter() {
         public void windowClosing( WindowEvent e ) {
            quit( 0 );
         }
      } );

      frameMain.pack();
      frameMain.setVisible( true );
   }

   void setupUserProperties() {
      String strFilename = _prpUser.getPrp( PRP_DIFF_LOAD_FILENAME0 );
      if ( strFilename == null )
         strFilename = ResourceApp.appFilename( "resources" + File.separator +
                                                "example-old.txt" );
      _modelFilename0 = new ModelFilename( strFilename );

      strFilename = _prpUser.getPrp( PRP_DIFF_LOAD_FILENAME1 );
      if ( strFilename == null )
         strFilename = ResourceApp.appFilename( "resources" + File.separator +
                                                "example-new.txt" );
      _modelFilename1 = new ModelFilename( strFilename );
   }

   void setupCommands() {
      CommandHolder cmdHolder = ResourceApp.cmdHolder;

      cmdHolder.addCommand( "select.old", new ActionListener() {
         public void actionPerformed( ActionEvent e ) {
            _modelFilename0.chooseFilename();
         }
      } );

      cmdHolder.addCommand( "select.new", new ActionListener() {
         public void actionPerformed( ActionEvent e ) {
            _modelFilename1.chooseFilename();
         }
      } );

      cmdHolder.addCommand( "quit", new ActionListener() {
         public void actionPerformed( ActionEvent e ) {
            quit( 0 );
         }
      } );

      cmdHolder.addCommand( "diff.save", new ActionListener() {
         public void actionPerformed( ActionEvent e ) {
            DiffHolder diffHolder =
                  new DiffHolder( _modelFilename0.text(),
                                  _modelFilename1.text() );
            diffSaveHtml( diffHolder );
         }
      } );

      cmdHolder.addCommand( "diff.show", new ActionListener() {
         public void actionPerformed( ActionEvent e ) {
            DiffHolder diffHolder =
                  new DiffHolder( _modelFilename0.text(),
                                  _modelFilename1.text() );
            diffShowHtml( diffHolder );
         }
      } );
   }

   public void diffSaveHtml( DiffHolder diffHolder ) {
      String strFileSaveHtml =
            _prpUser.getPrp( PRP_DIFF_SAVE_HTML_FILENAME, null );
      JFileChooser fileChooser = new JFileChooser();
      fileChooser.setDialogTitle(
            ResourceApp.getResString( "diff.save.filechooser" ) );
      if ( strFileSaveHtml != null )
         fileChooser.setSelectedFile( new File( strFileSaveHtml ) );
      int nOk = fileChooser.showSaveDialog( null );
      if ( nOk != JFileChooser.APPROVE_OPTION )   // Don't save
         return;

      File fileSaveHtml = fileChooser.getSelectedFile();
      _prpUser.setPrp( PRP_DIFF_SAVE_HTML_FILENAME,
                       fileSaveHtml.getAbsolutePath() );

      System.out.println( "Saving HTML output..." );
      try {
         PrintStream shtml = new PrintStream(
               new BufferedOutputStream(
                     new FileOutputStream( fileSaveHtml ) ) );
         diffHolder.buildHtmlChanges( shtml );
         shtml.close();
      } catch (IOException e) {
         System.err.println( "Failed to save Html output:" + e );
      }
      System.out.println( "Done!" );
   }

   public void diffShowHtml( DiffHolder diffHolder ) {
      System.out.println( "Creating HTML output..." );
      try {
         ByteArrayOutputStream os = new ByteArrayOutputStream();
         PrintStream shtml = new PrintStream(
               new BufferedOutputStream(
                     os ) );
         diffHolder.buildHtmlChanges( shtml );

         InputStream is = new ByteArrayInputStream( os.toByteArray() );
         shtml.close();
         os.close();

         System.out.println( "Creating HTML Viewer..." );

         JFrame frameEditor = new JFrame( "Diff result" );
         JEditorPane editor = new JEditorPane();
         JScrollPane paneScroll = new JScrollPane( editor );
         editor.setEditorKit( JEditorPane.createEditorKitForContentType( "text/html" ) );
         editor.setEditable( false );
         editor.read( is, new HTMLDocument() );
         JPanel panelContent = new JPanel();
         panelContent.setLayout( new BorderLayout() );
         panelContent.add( paneScroll, BorderLayout.CENTER );
         frameEditor.setContentPane( paneScroll );
         frameEditor.pack();
         frameEditor.setSize( 600, 400 );
         Dimension screen = frameEditor.getToolkit().getScreenSize();
         frameEditor.setLocation( ( screen.width - frameEditor.getSize().width ) / 2,
                                  ( screen.height - frameEditor.getSize().height ) / 2 );
         frameEditor.setVisible( true );
      } catch (IOException e) {
         System.err.println( "Failed creation Html output: " + e );
      }
      System.out.println( "Done!" );
   }

   public void quit( int nStatus ) {
      _prpUser.save();
      System.exit( nStatus );
   }

   public void quitInitFail() {
      System.exit( 2 );
   }

   public static void main( String args[] ) {
      new DiffWordApp();
   }

}


