package gaiacrtn.diffword;

import gaiacrtn.util.diff.DiffProcessor;

import java.util.List;
import java.io.PrintStream;

/**
 * Created by IntelliJ IDEA.
 * User: Ice Breaker
 * Date: 20 fvr. 2003
 * Time: 22:05:04
 * To change this template use Options | File Templates.
 */
public class HTMLDiffProcessor implements DiffProcessor {
   private List _originalMarkers;
   private List _modifiedMarkers;
   private int _originalIndex = 0;
   private int _modifiedIndex = 0;
   private int _previousChangeType = INVALID;
   private PrintStream _outputStream;
   private Printer _printer = new Printer();

   public HTMLDiffProcessor( List originalMarkers, List modifiedMarkers, PrintStream outputStream ) {
      _originalMarkers = originalMarkers;
      _modifiedMarkers = modifiedMarkers;
      _outputStream = outputStream;
      beginProcessing();
   }

   private void beginProcessing() {
      _outputStream.println( "<HTML><BODY><PRE>" );
   }

   public void processChange( int changeType ) {
      if ( changeType != _previousChangeType ) {
         endsChangeTypeTag( _previousChangeType );
         startsChangeTypeTag( changeType );
      }

      outputNextMarker( changeType );

      _previousChangeType = changeType;
   }

   public void finishProcessing() {
      endsChangeTypeTag( _previousChangeType );
      _outputStream.println();
      _outputStream.println( "</PRE></BODY></HTML>" );
   }

   private void startsChangeTypeTag( int changeType ) {
      String startTag;
      switch (changeType) {
      case INSERTED:
         startTag = "<B style=\"color:red\">";
         break;
      case DELETED:
         startTag = "<STRIKE style=\"color:blue\">";
         break;
      default:
         startTag = "";
         break;
      }
      _outputStream.print( startTag );
   }

   private void endsChangeTypeTag( int changeType ) {
      String endTag;
      switch (changeType) {
      case INSERTED:
         endTag = "</B>";
         break;
      case DELETED:
         endTag = "</STRIKE>";
         break;
      default:
         endTag = "";
         break;
      }
      _outputStream.print( endTag );
   }

   private void outputNextMarker( int changeType ) {
      TextMarker marker;
      switch (changeType) {
      case UNMODIFIED:
         marker = (TextMarker) _originalMarkers.get( _originalIndex );
         ++_originalIndex;
         ++_modifiedIndex;
         break;
      case INSERTED:
         marker = (TextMarker) _modifiedMarkers.get( _modifiedIndex++ );
         break;
      case DELETED:
         marker = (TextMarker) _originalMarkers.get( _originalIndex++ );
         break;
      default:
         return;
      }

      marker.apply( _printer );
   }

   private class Printer implements TextMarkerSwitch {
      public void caseWord( TextMarkerWord marker ) {
         _outputStream.print( escapeText( marker.toString() ) );
      }

      private String escapeText( String text ) {
         StringBuffer escaped = new StringBuffer();
         for ( int index =0; index < text.length(); ++index ) {
            char c = text.charAt( index );
            switch ( c )
            {
            case '<':
              escaped.append( "&lt;" );
              break;
            case '>':
              escaped.append( "&gt;" );
              break;
            case '&':
              escaped.append( "&amp;" );
              break;
            case '\'':
              escaped.append( "&apos;" );
              break;
            case '"':
              escaped.append( "&quot;" );
              break;
            case '\u00e9':    // '':
               escaped.append( "&eacute;" );
               break;
            case '\u00e8':    // '':
               escaped.append( "&egrave;" );
               break;
            case '\u00eb':    // '':
               escaped.append( "&euml;" );
               break;
            case '\u00e0':    // '':
               escaped.append( "&agrave;" );
               break;
            default:
              escaped.append( c );
            }
          }

         return escaped.toString();
       }

      public void caseEndLine( TextMarkerEndLine marker ) {
         if ( _previousChangeType != DELETED )
            _outputStream.println();
      }

   }
}
