/**
 * Title:        <p>
 * Description:  <p>
 * Copyright:    Copyright (c) <p>
 * Company:      <p>
 * @author
 * @version 1.0
 */
package gaiacrtn.dom;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.List;
import java.util.Vector;

public class NodeHelper {

   public interface NodeFilter {
      public boolean acceptNode( Node node );
   }

   public interface ElementFilter {
      public boolean acceptElement( Element elt );
   }

   static public class Element2NodeFilterAdapter implements NodeFilter {
      private ElementFilter _filter;

      Element2NodeFilterAdapter( ElementFilter filter ) {
         _filter = filter;
      }

      public boolean acceptNode( Node node ) {
         if ( node instanceof Element )
            return _filter.acceptElement( (Element) node );
         return false;
      }
   }

   public NodeHelper() {
   }

   /** Gets the list of all child nodes that are accepted by the specified
    * filter.
    * @param node The child node of this node which are accepted by the filter
    *             are returned in the list.
    * @param filter The method accept() is called for each child node.
    *               If \c true is returned then the node is added to the list.
    * @return List that contains the child element (node of type ELEMENT_TYPE)
    *         of the specified node.
    */
   static public List subNodes( NodeList listNode, NodeFilter filter ) {
      List listSubNode = new Vector();
      int nNodeCount = listNode.getLength();
      for (int nIdx = 0; nIdx < nNodeCount; ++nIdx) {
         Node nodeIt = listNode.item( nIdx );
         if ( filter.acceptNode( nodeIt ) ) {
            listSubNode.add( nodeIt );
         }
      }
      return listSubNode;
   }

   /** Gets the list of all child elements node of the specified node.
    * @param node The child element list of this node is returned.
    * @return List that contains the child element
    *         (node of type org.w3c.dom.Node.ELEMENT_NODE) of the specified node.
    */
   static public List subElements( NodeList listNode ) {
      return subNodes( listNode,
                       new NodeFilter() {
                          public boolean acceptNode( Node node ) {
                             return node.getNodeType() == Node.ELEMENT_NODE;
                          }
                       } );
   }

   /** Gets the list of all child elements that match a given criteria
    *  of the specified node.
    * @param node The child element list of this node is returned.
    * @param filter The method accept() is called for each child element.
    *               If \c true is returned then the element is added to the list.
    * @return List that contains the child element
    *         (node of type org.w3c.dom.Node.ELEMENT_NODE) of the specified node.
    */
   static public List subElements( NodeList nodeList, ElementFilter filter ) {
      return subNodes( nodeList,
                       new Element2NodeFilterAdapter( filter ) );
   }

   /** Gets the list of all child elements that have the specified tag name.
    * @param listElement List of elements that is searched.
    *             name are returned in the list.
    * @param strAttributeName Name of the attribute that is used as a search
    *                         criteria.
    * @param strAttributeValue Value of the attribute of the searched element.
    * @return Searched element. \c null if none is found.
    */
   static public Element findElement( NodeList listElement,
                                      String strAttributeName,
                                      String strAttributeValue ) {
      int nNodeCount = listElement.getLength();
      for (int nIdx = 0; nIdx < nNodeCount; ++nIdx) {
         Node nodeIt = listElement.item( nIdx );
         if ( nodeIt instanceof Element ) {
            Element elt = (Element) nodeIt;
            if ( strAttributeValue.equals( elt.getAttribute( strAttributeName ) ) )
               return elt;
         }
      }
      return null;
   }
}