/**
 * Title:        <p>
 * Description:  <p>
 * Copyright:    Copyright (c) <p>
 * Company:      <p>
 * @author
 * @version 1.0
 */
package gaiacrtn.swing;

import gaiacrtn.dom.NodeHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.NoSuchElementException;

public class MenuDefFactoryXMenu implements MenuDefFactory {
   private Document _dom;
   private Element _eltMenuList;

   /** Default menu factory.
    * If a menu can not be created by this menu factory (definition not in
    * the XML document), then this default factory is asked to create the
    * menu.
    */
   private MenuDefFactory _factoryMenuDefault;

   public MenuDefFactoryXMenu( File fileXMenu )
         throws IOException {

      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating( true );
      try {
         DocumentBuilder builder = factory.newDocumentBuilder();
         // Set validation error handler...
         builder.setErrorHandler(
               new org.xml.sax.ErrorHandler() {
                  // ignore fatal errors (an exception is guaranteed)
                  public void fatalError( SAXParseException exception )
                        throws SAXException {
                  }

                  // treat validation errors as fatal
                  public void error( SAXParseException e )
                        throws SAXParseException {
                     throw e;
                  }

                  // dump warnings too
                  public void warning( SAXParseException err )
                        throws SAXParseException {
                     System.err.println( "** Warning"
                                         + ", line " + err.getLineNumber()
                                         + ", uri " + err.getSystemId() );
                     System.err.println( "   " + err.getMessage() );
                  }
               }
         );

         _dom = builder.parse( fileXMenu );

         _eltMenuList = _dom.getDocumentElement();
      } catch (SAXParseException e) {
         // Error generated by the parser
         System.out.println( "\n** Parsing error"
                             + ", line " + e.getLineNumber()
                             + ", uri " + e.getSystemId() );
         System.out.println( "   " + e.getMessage() );
         // Use the contained exception, if any
         Exception x = e;
         if ( e.getException() != null )
            x = e.getException();
         x.printStackTrace();
      } catch (SAXException e) {
         // Error generated by this application
         // (or a parser-initialization error)
         Exception x = e;
         if ( e.getException() != null )
            x = e.getException();
         x.printStackTrace();
      } catch (ParserConfigurationException e) {
         // Parser with specified options can't be built
         e.printStackTrace();
      }
   }

   public void setDefaultMenuDefFactory( MenuDefFactory factoryMenuDefault ) {
      _factoryMenuDefault = factoryMenuDefault;
   }

   public MenuDefFactory defaultMenuDefFactory() {
      return _factoryMenuDefault;
   }

   protected Element findMenu( String strMenuName ) {
      NodeList listEltMenu = _eltMenuList.getElementsByTagName( XMenu.TAG_MENU );
      Element eltMenu = NodeHelper.findElement( listEltMenu,
                                                XMenu.ATT_MENU_NAME,
                                                strMenuName );
      return eltMenu;
   }

   public MenuDefManager findMenuManager( String strMenuName ) {
      Element eltMenu = findMenu( strMenuName );
      if ( eltMenu != null )
         return new XMenuDefManager( eltMenu );
      if ( defaultMenuDefFactory() != null )
         return defaultMenuDefFactory().findMenuManager( strMenuName );
      return null;
   }


   protected class XMenuDefManager implements MenuDefManager {
      protected Element _eltMenu;
      protected List _listMenuElement;

      XMenuDefManager( Element eltMenu ) {
         _eltMenu = eltMenu;
         _listMenuElement = NodeHelper.subElements( _eltMenu.getChildNodes() );
      }

      public MenuDefIterator iterator() {
         return new XMenuDefIterator();
      }

      public String name() {
         return _eltMenu.getAttribute( XMenu.ATT_MENU_NAME );
      }


      protected class XMenuDefIterator implements MenuDefIterator {
         private int _nPos = 0;

         public int size() {
            return _listMenuElement.size();
         }

         public void begin() {
            _nPos = 0;
         }

         public boolean hasNext() {
            return _nPos < size();
         }

         public MenuDefEntry next() {
            if ( !hasNext() )
               throw new NoSuchElementException();

            Element elt = (Element) _listMenuElement.get( _nPos++ );
            String strTagName = elt.getTagName();
            if ( strTagName == XMenu.TAG_MENUITEM )
               return new XMenuDefEntryMenuItem( elt );
            else if ( strTagName == XMenu.TAG_MENUSEP )
               return new MenuDefEntrySeparator();
            else if ( strTagName == XMenu.TAG_SUBMENU )
               return new XMenuDefEntrySubMenu( elt );

            System.err.println(
                  "  Warning: unknown element type in XML menu definition for\n" +
                  "menu \"" + name() + "\", entry #" + ( _nPos + 1 ) + "." );
            return null;
         }
      }
   }


   protected class XMenuDefEntryMenuItem extends MenuDefEntryMenuItem {
      private Element _eltMenuItem;

      XMenuDefEntryMenuItem( Element eltMenuItem ) {
         _eltMenuItem = eltMenuItem;
      }

      public String command() {
         return _eltMenuItem.getAttribute( XMenu.ATT_MENUITEM_CMD );
      }
   }


   protected class XMenuDefEntrySubMenu extends MenuDefEntrySubMenu {
      private Element _eltSubMenu;

      XMenuDefEntrySubMenu( Element eltSubMenu ) {
         _eltSubMenu = eltSubMenu;
      }

      public String name() {
         return _eltSubMenu.getAttribute( XMenu.ATT_SUBMENU_NAME );
      }
   }
}


interface XMenu {
   final static public String TAG_MENU = "menu";
   final static public String ATT_MENU_NAME = "name";
   final static public String TAG_SUBMENU = "submenu";
   final static public String ATT_SUBMENU_NAME = "name";
   final static public String TAG_MENUITEM = "menuitem";
   final static public String ATT_MENUITEM_CMD = "cmd";
   final static public String TAG_MENUSEP = "menusep";
}
