/**
 * Title:        Diff Word<p>
 * Description:  A diff that is not line based but word based.<p>
 * Copyright:    Copyright (c) Baptiste Lepilleur<p>
 * Company:      Gaia Creation<p>
 * @author Baptiste Lepilleur
 * @version 1.0
 */
package gaiacrtn.swing.event;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.util.Iterator;
import java.util.Vector;

public class ChangeEventSupport {
   private Object _objSource;
   private Vector _listListener;
   private ChangeEvent _event;

   /** Constructs a ChangeEventSupport object.
    * @param objSource The object to be given as the source for any events.
    */
   public ChangeEventSupport( Object objSource ) {
      _objSource = objSource;
   }

   /** Adds a ChangeListener to the listener list.
    * @param listener  The ChangeListener to be added
    */
   public void addChangeListener( ChangeListener listener ) {
      if ( _listListener == null ) {
         _listListener = new Vector();
      }

      _listListener.addElement( listener );
   }

   /** Removes a ChangeListener from the listener list.
    * @param listener  The ChangeListener to be removed
    */
   public void removeChangeListener( ChangeListener listener ) {
      if ( _listListener == null )
         return;

      _listListener.removeElement( listener );
   }

   /** Fire an existing ChangeEvent to any registered listeners.
    * @param event  The ChangeEvent object.
    */
   public void fireChange() {
      if ( _listListener != null ) {
         if ( _event == null )
            _event = new ChangeEvent( _objSource );

         Iterator it = _listListener.iterator();
         while (it.hasNext()) {
            ChangeListener target = (ChangeListener) it.next();
            target.stateChanged( _event );
         }
      }
   }
}
