package gaiacrtn.util;


/** This interface must extended by all switches.
 * The {@link gaiacrtn.util.Switchable} and {@link gaiacrtn.util.Switch} interfaces
 * are extensions of the design patterns known as "Visitor".
 * <p>
 * A <code>Switch</code> implements a set of methods that are called by
 * {@link gaiacrtn.util.Switchable} objects.
 * <p>
 * The following code is used to provide a simple way to process object
 * differently in function of their types:
 * <pre>
 * public interface SwitchStatement extends Switch
 * {
 *   void caseIf( StatementIf statement );
 *   void caseFor( StatementFor statement );
 *   void caseWhile( StatementWhile statement );
 * }
 * </pre>
 * That means that the objects of type <code>StatementIf</code>,
 * <code>StatementFor</code>, <code>StatementWhile</code> would need to
 * be {@link gaiacrtn.util.Switchable Switchable} objects.
 * <p>
 * The interesting thing about that thing is that you can write an algorithm
 * that, for example, walk through object and call the
 * {@link gaiacrtn.util.Switchable#apply} method on them, therefore providing
 * a way to process the object differently in function of the <code>Switch</code>
 * that was given to the algorithm.
 * <p>
 * Imagine you wrote a code parser that would walk through the code and
 * recognizes objects. The parser would switch the object, therefore processing
 * it as specified by the switch implementing. You could have a switch that
 * compute some statistic, another one that generates documentations...
 * @see {@link gaiacrtn.util.Switchable}
 * @see gaiacrtn.util.Switchable
 */
public interface Switch {
}