/**
 * Title:        Diff Word<p>
 * Description:  A diff that is not line based but word based.<p>
 * Copyright:    Copyright (c) Baptiste Lepilleur<p>
 * Company:      Gaia Creation<p>
 * @author Baptiste Lepilleur
 * @version 1.0
 */
package gaiacrtn.util;

import java.io.*;

/** Classes used to read a text line by line and process each line.
 */
public class TextReader {

   /** Interface used to process each line of the text.
    */
   public interface LineProcessor {

      /** Process a line of text.
       * @param strLine Text of this line.
       * @param nLineNo Line number (0 is the first line).
       */
      public void processLine( String strLine, int nLineNo );
   }

   /** Reads a text from a file.
    * @param strFilename Filename of the file that contains the text.
    * @param lineProcessor Object that processes each line of the text.
    */
   static public void read( String strFilename,
                            TextReader.LineProcessor lineProcessor ) {
      try {
         read( new InputStreamReader( new FileInputStream( strFilename ),
                                      "ISO-8859-1" ),
               lineProcessor );   // code page 850: texte encod dos franais
      } catch (IOException e) {
      }
   }

   /** Reads a text from a Reader.
    * @param reader Reader from which the text is read.
    * @param lineProcessor Object that processes each line of the text.
    */
   static public void read( Reader reader,
                            TextReader.LineProcessor lineProcessor ) {
      try {
         LineNumberReader readerLine = new LineNumberReader( reader );

         String strLine;
         strLine = readerLine.readLine();

         while (strLine != null) {
            lineProcessor.processLine( strLine, readerLine.getLineNumber() );
            strLine = readerLine.readLine();
         }
      } catch (IOException exc) {
      }
   }
}
